import { useState, useContext, useEffect } from 'react';
import SearchContext from './search-context.js';
import { DEBOUNCE_QUERY_UPDATE, DEBOUNCE_SEARCH, FULL_SEARCH, QUICK_SEARCH } from './search-const.js';
let updateQueryTimeout = 0;
let searchTimeout = 0;

const useSearch = (inputValue = '', maxHits = 25) => {
  const [hits, setHits] = useState([]);
  const [totalHits, setTotalHits] = useState(0);
  const [isLoading, setIsLoading] = useState(false);
  const [queryId, setQueryId] = useState('');
  const [placeholder, setPlaceholder] = useState('');
  const {
    matching,
    updateQueryString,
    type,
    search,
    setSearchQueryState
  } = useContext(SearchContext);
  const isFullSearch = type === FULL_SEARCH;
  const isQuickSearch = type === QUICK_SEARCH;

  const debouncedSearch = () => {
    if (updateQueryTimeout) {
      clearTimeout(updateQueryTimeout);
    }

    if (searchTimeout) {
      clearTimeout(searchTimeout);
    }

    updateQueryTimeout = window.setTimeout(() => {
      if (isQuickSearch) {
        setSearchQueryState(inputValue);
      } else if (isFullSearch) {
        updateQueryString(inputValue, type);
      }
    }, DEBOUNCE_QUERY_UPDATE);
    searchTimeout = window.setTimeout(async () => {
      if (inputValue) {
        setIsLoading(true);

        try {
          const results = await search(inputValue, matching, maxHits);

          if (results.hits.length === 0) {
            setPlaceholder(`We’re sorry! We could’t find results for «${inputValue}»`);
          } else {
            setPlaceholder('');
          }

          setHits(results.hits);
          setTotalHits(results.totalHits);

          if (results.queryId) {
            setQueryId(results.queryId);
          }
        } catch (error) {
          console.error(error);
        } finally {
          setIsLoading(false);
        }
      } else {
        // Clear hits and placeholder when search string is empty.
        // It prevents show previous negative result when searching from scratch
        setPlaceholder('');
        setHits([]);
      }
    }, DEBOUNCE_SEARCH);
  };

  useEffect(() => {
    debouncedSearch();
    return () => clearTimeout(updateQueryTimeout);
  }, [inputValue, matching]);
  return {
    hits,
    totalHits,
    isLoading,
    queryId,
    placeholder
  };
};

export { useSearch };
